;
; -*-scheme-*-
;
; Script-Fu Charcoal v1.0
; A Gimp script which turns a photo into a black and white image which can look 
; quite like a charcoal drawing. Works on RGB, Indexed or Greyscale images.
;
; Tested with Gimp 2.6
;
; Micomicon - February 2011
;
; This software is in the public domain. 
;
; I have released this script into the wild in the hope that it may 
; prove useful. It comes with absolutely no warranty whatsoever, 
; without even the implied warranty of merchantability or fitness 
; for any particular purpose. See the GNU General Public License for 
; details. You may redistribute and/or modify this script or extract 
; segments from it without prior consent.
;
; The script is located in "<Image> / Filters / Artistic / Charcoal..."



(define (script-fu-charcoal image drawable granuality lowpoint highpoint boost? invert? resize? papercolour? chalk?)

    (let* (

                (new-width 1200)
                (new-height 1200) 
                (ratio 0)
                (width  (car  (gimp-drawable-width drawable)))
		(height (car  (gimp-drawable-height drawable)))
                (copy-layer -1)
                (charcoal-layer -1)
                (chalk-layer -1)                                            

    )


    ;Save the current settings
    (gimp-context-push)

    ; Start an undo group so the image can be restored with one undo
    (gimp-image-undo-group-start image)

    
    ; Merge visible layers and clear any selections

    (gimp-image-flatten image)
    (gimp-selection-none image)



    ; Rescale the image to the ideal size for optimum effect

           (when (equal? resize? TRUE)
           (begin

           ; establish parameters of image dimensions
                
           (set! ratio (/ height width)) 


           ; set new dimensions if portrait

           (if (> height width) 
           (set! new-height (round (* 960 ratio))))
           (if (> height width)  
           (set! new-width 960))
                
           ; set new dimensions if landscape

           (if (< height width) 
           (set! new-height (round (* 1280 ratio))))
           (if (< height width)  
           (set! new-width 1280))
                
           ; leave at defaults if square.

           ; rescale image to new dimensions.
           (gimp-image-scale image new-width new-height) ))
	     

    (set! drawable (car (gimp-image-get-active-drawable image)))

    ; Check and convert image to indexed if necessary

    (if (= (car (gimp-drawable-is-rgb drawable)) 0) 
    (gimp-image-convert-rgb image))


    ; Increase the contrast if required

         (when (equal? boost? TRUE)
         (begin

         (gimp-levels-stretch drawable)) )


    ; Add some texture
    (plug-in-sharpen RUN-NONINTERACTIVE image drawable granuality)

    ; Creating chalk highlights

         (when (equal? chalk? TRUE)
         (begin

         ; Copy background layer
         (set! charcoal-layer (car (gimp-image-get-active-layer image)))
         (set! copy-layer (car (gimp-layer-copy charcoal-layer TRUE)) ) 
         (gimp-image-add-layer image copy-layer -1)

         ; Add some white (50-75 = Range for white highlights)
         (gimp-image-set-active-layer image copy-layer)
         (set! drawable (car (gimp-image-get-active-drawable image)))
         (gimp-threshold drawable 50 75)

         ; Copy to a new transparent layer

         (gimp-by-color-select drawable '(255 255 255) 0 
          CHANNEL-OP-REPLACE FALSE FALSE 0 FALSE) 
         (gimp-edit-cut drawable)

         (set! chalk-layer (car (gimp-layer-new  image  width  height   
          RGBA-IMAGE "Chalk Layer" 100 NORMAL-MODE)))     
         (gimp-image-add-layer image chalk-layer -1) 
        
         (set! drawable (car (gimp-image-get-active-drawable image)))
         (gimp-drawable-fill chalk-layer TRANSPARENT-FILL)

         (set! chalk-layer (car (gimp-image-get-selection image)))     
         (car (gimp-drawable-is-layer drawable)) 
         (gimp-image-set-active-layer image charcoal-layer)
         (gimp-floating-sel-to-layer (car (gimp-edit-paste drawable 1)))
         (gimp-selection-none image)


         ; Switch back to charcoal layer and move above scrap
         (gimp-image-set-active-layer image charcoal-layer)
         (set! drawable (car (gimp-image-get-active-drawable image)))
         (gimp-image-raise-layer image charcoal-layer)
         (gimp-image-raise-layer image charcoal-layer)  ))


    ; Set the threshold    
    (gimp-threshold drawable lowpoint highpoint)


    ; Retain negative image if required  

         (when (equal? invert? FALSE)
         (begin

         (gimp-invert drawable)) )

    ; Choose a paper colour

         (when (equal? papercolour? TRUE)
         (begin

         (plug-in-colorify RUN-INTERACTIVE image drawable '(255 255 255)) ) )

    ; Convert lowlights to charcoal grey   

         (when (equal? boost? FALSE)
         (begin
       
         (gimp-levels drawable 0 000 255 1 035 255) ) )


    ; Merge chalk and charcoal layers
    (gimp-image-merge-visible-layers image 0)

    ; Restore previous settings
    (gimp-context-pop)  

    ; Finish the undo group for the process
    (gimp-image-undo-group-end image)

    ; Display the updated image
    (gimp-displays-flush)

    
    )
)


(script-fu-register "script-fu-charcoal"
            _"<Image>/Filters/Artistic/Charcoal..."
            "Create a charcoal drawing from a photo"
            "Micomicon"
            "Adrian Spriddell"
            "February 2011"
            "*"
            SF-IMAGE		"Image"     0
            SF-DRAWABLE		"Drawable"  0
	    SF-ADJUSTMENT       "Granuality"      '(65 0 90 1 1 0 0)
	    SF-ADJUSTMENT       "Lowlights crossover point"      '(70 0 127 1 1 0 0)
	    SF-ADJUSTMENT       "Highlights crossover point"     '(170 127 255 1 1 0 0)
            SF-TOGGLE           "Boost contrast" 0 
            SF-TOGGLE           "Invert to negative" 0  
            SF-TOGGLE           "Resize image for optimum effect" 1 
	    SF-TOGGLE           "Coloured paper" 0  
	    SF-TOGGLE           "Add chalk highlights" 0  
      
)
